/*
 *	Ohio Trollius
 *	Copyright 1995 The Ohio State University
 *	GDB
 *
 *	$Log:	lamclean.c,v $
 * Revision 6.1  96/11/23  18:27:25  nevin
 * Ohio Release
 * 
 * Revision 6.0  96/02/29  13:20:14  gdburns
 * Ohio Release
 * 
 * Revision 5.2.1.2  96/01/19  16:05:08  gdburns
 * Add trace daemon cleanup.
 * Spin print nodeids.
 * 
 * Revision 5.2.1.1  95/06/09  15:14:27  raja
 * Remove groupd cleaning.
 * 
 * Revision 5.2  94/08/22  13:42:50  gdburns
 * Ohio Release
 * 
 * Revision 5.1.1.1  94/08/22  11:51:27  gdburns
 * Add robustness.
 * 
 * Revision 5.1  94/05/18  10:39:01  gdburns
 * Ohio Release
 * 
 * Revision 2.3  94/05/18  10:08:06  gdburns
 * new for 5.1
 * 
 *	Function:	- command to clean out several daemons on all nodes
 */

#include <stdio.h>
#include <stdlib.h>

#include <args.h>
#include <debug.h>
#include <freq.h>
#include <ksignal.h>
#include <net.h>
#include <preq.h>
#include <priority.h>
#include <terror.h>
#include <trreq.h>
#include <typical.h>

/*
 * local functions
 */
static void		help();


main(argc, argv)

int			argc;
char			*argv[];

{
	int4		*all;		/* array of all node IDs */
	int4		i;
	int4		nall;		/* count of all nodes */
	int		fl_verbose;	/* verbosity flag */
/*
 * Parse command-line arguments.
 */
	validopts("hv");

	if (do_args(&argc, argv) || (argc != 1)) {
		fprintf(stderr, "lamclean [-hv]\n");
		exit(EUSAGE);
	}

	if (opt_taken('h')) {
		help();
		exit(0);
	}

	fl_verbose = opt_taken('v');
/*
 * Initialize and attach to Trollius kernel.
 */
	if (kinit(PRCMD)) {
		terror("lamclean (kinit)");
		exit(errno);
	}
/*
 * Get an array of all node IDs.
 */
	nall = getnall();
	all = (int4 *) malloc((unsigned) (nall * sizeof(int4)));

	if (all == 0) lamfail("lamclean (malloc)");

	if (getall(all, nall)) lamfail("lamclean (getall)");
/*
 * Doom all appl processes.
 */
	if (fl_verbose) nodespin_init("killing processes,");

	for (i = 0; i < nall; ++i) {

		if (fl_verbose) nodespin_next(all[i]);

		if (rpdoom(all[i], SELECT_APPL, 0, SIGUDIE)) {
			lamfail("lamclean (rpdoom)");
		}
	}
/*
 * Reset buffers.
 */
	sleep(1);
	if (fl_verbose) {
		nodespin_end();
		nodespin_init("sweeping messages,");
	}

	for (i = 0; i < nall; ++i) {

		if (fl_verbose) nodespin_next(all[i]);

		if (rbfwipe(all[i])) {
			lamfail("nlamclean (rbfwipe)");
		}
	}
/*
 * Reset file daemons.
 */
	if (fl_verbose) {
		nodespin_end();
		nodespin_init("closing files,");
	}

	for (i = 0; i < nall; ++i) {

		if (fl_verbose) nodespin_next(all[i]);

		if (rfrmfd(all[i], NOTFD)) {
			lamfail("lamclean (rfrmfd)");
		}
	}
/*
 * Reset trace deamons.
 */
	if (fl_verbose) {
		nodespin_end();
		nodespin_init("sweeping traces,");
	}

	for (i = 0; i < nall; ++i) {

		if (fl_verbose) nodespin_next(all[i]);

		if (lam_rtrsweep(all[i], TRANYLIST)) {
			lamfail("lamclean (lam_rtrsweep)");
		}
	}

	if (fl_verbose) nodespin_end();

	kexit(0);
	return(0);
}

/*
 *	help
 *
 *	Function:	- prints helpful information on this command
 */
static void
help()

{
	printf("\nSynopsis:	lamclean [options]\n");
	printf("\nDescription:	Clean up resources on all nodes.\n");
	printf("\nOptions:	-h\t\tPrint this help message.\n");
	printf("\t\t-v\t\tBe verbose.\n");
}
